//**********************
// LCD module Function for HD44780 compatible controller(KS0066)
// 
//  8 characters x 1 line  : SC0801
//  8 characters x 2 line  : SC0802
// 10 characters x 1 line  : SC1001
// 10 characters x 4 line  : SC1004
// 16 characters x 1 line  : SC1601, L1671
// 16 characters x 2 lines : SC1602, SD1602, L1672, L1682, L1692
// 16 characters x 4 lines : SC1604, L1634
// 20 characters x 2 lines : SC2002, L2032
// 20 characters x 4 lines : SC2004, L2034
// 24 characters x 2 lines : SC2402, L2462
// 40 characters x 2 lines : SC4002, L4052
// 40 characters x 4 lines : SC4004, L4044	not yet implementation
// 
// Condition:
// 
// (C)Copyright 2009 All rights reserved by Y.Onodera
// http://einst.hp.infoseek.co.jp
//**********************
#if defined(__dsPIC30F__)
#include <p30fxxxx.h>
#elif defined(__dsPIC33F__)
#include <p33Fxxxx.h>
#elif defined(__PIC24H__)
#include <p24Hxxxx.h>
#elif defined(__PIC24F__)
#include <p24Fxxxx.h>
#endif

#include "Wait.h"
#include "LCD.h"


void PutLCD(int rs, char a)
{

	RW_PIN = 0;		// to write
	// 0:command, 1:data
	RS_PIN = rs;	// Select command or data
	
	// set pins as output
	TRIS_DATA_PIN_7 = 0;
	TRIS_DATA_PIN_6 = 0;
	TRIS_DATA_PIN_5 = 0;
	TRIS_DATA_PIN_4 = 0;
	
	DATA_PIN_7 = (unsigned int)((a & 0x80)>>7);
	DATA_PIN_6 = (unsigned int)((a & 0x40)>>6);
	DATA_PIN_5 = (unsigned int)((a & 0x20)>>5);
	DATA_PIN_4 = (unsigned int)((a & 0x10)>>4);
	
# ifdef EIGHT_BIT_INTERFACE
	// set pins as output
	TRIS_DATA_PIN_3 = 0;
	TRIS_DATA_PIN_2 = 0;
	TRIS_DATA_PIN_1 = 0;
	TRIS_DATA_PIN_0 = 0;
		
	DATA_PIN_3 = (unsigned int)((a & 0x08)>>3);
	DATA_PIN_2 = (unsigned int)((a & 0x04)>>2);
	DATA_PIN_1 = (unsigned int)((a & 0x02)>>1);
	DATA_PIN_0 = (unsigned int)(a & 0x01);
		
	// pulse 220ns - 450ns
	E_PIN = 1;
	WaitNus(1);
	E_PIN = 0;
# else
	// pulse 220ns - 450ns
	E_PIN = 1;
	WaitNus(1);
	E_PIN = 0;
		
	RW_PIN = 0;
	RS_PIN = rs;
		
	DATA_PIN_7 = (unsigned int)((a & 0x08)>>3);
	DATA_PIN_6 = (unsigned int)((a & 0x04)>>2);
	DATA_PIN_5 = (unsigned int)((a & 0x02)>>1);
	DATA_PIN_4 = (unsigned int)(a & 0x01);
		
	// pulse 220ns - 450ns
	E_PIN = 1;
	WaitNus(1);
	E_PIN = 0;
#endif

	// wait 40us or 1.64ms
	if(rs==0 && (a==CLEAR_DISPLAY || a==RETURN_HOME) )
		// wait 1.64ms
		WaitNms(3);
	else
		WaitNus(100);

	// set pins as input
	TRIS_DATA_PIN_7 = 1;
	TRIS_DATA_PIN_6 = 1;
	TRIS_DATA_PIN_5 = 1;
	TRIS_DATA_PIN_4 = 1;
# ifdef EIGHT_BIT_INTERFACE
	TRIS_DATA_PIN_3 = 1;
	TRIS_DATA_PIN_2 = 1;
	TRIS_DATA_PIN_1 = 1;
	TRIS_DATA_PIN_0 = 1;
#endif

	RS_PIN = 0; 

}


char GetLCD(int rs)
{
	char data=0;
	unsigned char temp[8],i;

	// Set pins as input
	TRIS_DATA_PIN_7 = 1;
	TRIS_DATA_PIN_6 = 1;
	TRIS_DATA_PIN_5 = 1;
	TRIS_DATA_PIN_4 = 1;

# ifdef EIGHT_BIT_INTERFACE
	TRIS_DATA_PIN_3 = 1;
	TRIS_DATA_PIN_2 = 1;
	TRIS_DATA_PIN_1 = 1;
	TRIS_DATA_PIN_0 = 1;
#endif

	RW_PIN = 1;		// to read
	RS_PIN = rs;	// Select command or data
	WaitNus(1);
	E_PIN = 1;		// enable
	WaitNus(1);
	temp[7] = (char) READ_PIN_7;
	temp[7] = temp[7] << 7;
	temp[6] = (char) READ_PIN_6;
	temp[6] = temp[6] << 6;
	temp[5] = (char) READ_PIN_5;
	temp[5] = temp[5] << 5;
	temp[4] = (char) READ_PIN_4;
	temp[4] = temp[4] << 4;

#ifdef EIGHT_BIT_INTERFACE
	temp[3]  = (char) READ_PIN_3;
	temp[3] = temp[3] << 3;
	temp[2]  = (char) READ_PIN_2;
	temp[2] = temp[2] << 2;
	temp[1]  = (char) READ_PIN_1;
	temp[1] = temp[1] << 1;
	temp[0]  = (char) READ_PIN_0;
	E_PIN =0;		// disable
#else
	E_PIN = 0;		// disable
	RW_PIN = 1;		// to read
	RS_PIN = rs;	// Select command or data
	E_PIN = 1;		// enable
	WaitNus(1);

	temp[3] = (char) READ_PIN_7;
	temp[3] = temp[3] << 3;
	temp[2] = (char) READ_PIN_6;
	temp[2] = temp[2] << 2;
	temp[1] = (char) READ_PIN_5;
	temp[1] = temp[1] << 1;
	temp[0] = (char) READ_PIN_4;

	E_PIN = 0;	// disable
#endif

	for (i=0;i<8;i++)
	{
		data |= temp[i];
	}

	// wait 40us
	WaitNus(100);

	return data;
}



void InitLCD()
{
	// Wait 15ms
	WaitNms(30);

	// Initialize port config
//	CFG_DATA_PIN_7 = 1;
//	CFG_DATA_PIN_6 = 1;
//	CFG_DATA_PIN_5 = 1;
//	CFG_DATA_PIN_4 = 1;
	
	// set port zero
	DATA_PIN_7 = 0;
	DATA_PIN_6 = 0;
	DATA_PIN_5 = 0;
	DATA_PIN_4 = 0;
	
# ifdef EIGHT_BIT_INTERFACE
	DATA_PIN_3 = 0;
	DATA_PIN_2 = 0;
	DATA_PIN_1 = 0;
	DATA_PIN_0 = 0;
#endif
	
	// Initialize port config
//	CFG_E_PIN = 1;
//	CFG_RW_PIN = 1;
//	CFG_RS_PIN = 1;

	// set port zero
	RW_PIN = 0;
	RS_PIN = 0;
	E_PIN = 0;
	
	// set data port as output
	TRIS_DATA_PIN_7 = 0;
	TRIS_DATA_PIN_6 = 0;
	TRIS_DATA_PIN_5 = 0;
	TRIS_DATA_PIN_4 = 0;
	
#ifdef EIGHT_BIT_INTERFACE
	TRIS_DATA_PIN_3 = 0;
	TRIS_DATA_PIN_2 = 0;
	TRIS_DATA_PIN_1 = 0;
	TRIS_DATA_PIN_0 = 0;
#endif
	
	// set control pins as output
	TRIS_RW = 0;
	TRIS_RS = 0;
	TRIS_E = 0;

 	// Function Set
	DATA_PIN_7 = 0;
	DATA_PIN_6 = 0;
	DATA_PIN_5 = 1;
	DATA_PIN_4 = 1;
#ifdef EIGHT_BIT_INTERFACE
	DATA_PIN_3 = 0;
	DATA_PIN_2 = 0;
	DATA_PIN_1 = 0;
	DATA_PIN_0 = 0;
#endif

	// pulse 220ns - 450ns
	E_PIN = 1;
	WaitNus(1);
	E_PIN = 0;
	
	// wait 4.1ms
	WaitNms(8);

 	// Function Set
	DATA_PIN_7 = 0;
	DATA_PIN_6 = 0;
	DATA_PIN_5 = 1;
	DATA_PIN_4 = 1;
#ifdef EIGHT_BIT_INTERFACE
	DATA_PIN_3 = 0;
	DATA_PIN_2 = 0;
	DATA_PIN_1 = 0;
	DATA_PIN_0 = 0;
#endif
	
	// pulse 220ns - 450ns
	E_PIN = 1;
	WaitNus(1);
	E_PIN = 0;
	
	// wait 100us
	WaitNus(200);

 	// Function Set
	DATA_PIN_7 = 0;
	DATA_PIN_6 = 0;
	DATA_PIN_5 = 1;
	DATA_PIN_4 = 1;
#ifdef EIGHT_BIT_INTERFACE
	DATA_PIN_3 = 0;
	DATA_PIN_2 = 0;
	DATA_PIN_1 = 0;
	DATA_PIN_0 = 0;
#endif
	
	// pulse 220ns - 450ns
	E_PIN = 1;
	WaitNus(1);
	E_PIN = 0;
	
#ifdef EIGHT_BIT_INTERFACE
 	// Function Set for 8 bits mode
	DATA_PIN_7 = 0;
	DATA_PIN_6 = 0;
	DATA_PIN_5 = 1;
	DATA_PIN_4 = 1;
	DATA_PIN_3 = 1;
	DATA_PIN_2 = 0;
	DATA_PIN_1 = 0;
	DATA_PIN_0 = 0;
#else
 	// Function Set for 4 bits mode
	DATA_PIN_7 = 0;
	DATA_PIN_6 = 0;
	DATA_PIN_5 = 1;
	DATA_PIN_4 = 0;
#endif
	
	// pulse 220ns - 450ns
	E_PIN = 1;
	WaitNus(1);
	E_PIN = 0;
	
	// wait 40us
	WaitNus(100);

	// Function Set as INIT_MODE
	PutCmdLCD(INIT_MODE);
	  
	// Display ON/OFF
	PutCmdLCD(DISPLAY_ON & BLINK_OFF & CURSOR_ON);
	
	// Display Clear
	PutCmdLCD(CLEAR_DISPLAY);

	// Entry Mode Set
	PutCmdLCD(SHIFT_OFF & INC_MODE);

}


void ClsLCD()
{

	PutCmdLCD(CLEAR_DISPLAY);
	PutCmdLCD(RETURN_HOME);

}


//  8x1 = 00 01 02 03 04 05 06 07
//  8x2 = 00 01 02 03 04 05 06 07
//        40 41 42 43 44 45 46 47
// 10x1 = 00 01 02 03 04 05 06 07 08 09
// 10x4 = 00 01 02 03 04 05 06 07 08 09
//        40 41 42 43 44 45 46 47 48 49
//        0A 0B 0C 0D 0E 0F 10 11 12 13
//        4A 4B 4C 4D 4E 4F 50 51 52 53
// 16x1 = 00 01 02 03 04 05 06 07 40 41 42 43 44 45 46 47
// 16x2 = 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F
//        40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F
// 16x4 = 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F
//        40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F
//        10 11 12 13 14 15 16 17 18 19 1A 1B 1C 1D 1E 1F
//        50 51 52 53 54 55 56 57 58 59 5A 5B 5C 5D 5E 5F
// 20x2 = 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 13
//        40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53
// 20x4 = 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 13
//        40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53
//        14 15 16 17 18 19 1A 1B 1C 1D 1E 1F 20 21 22 23 24 25 26 27
//        54 55 56 57 58 59 5A 5B 5C 5D 5E 5F 60 61 62 63 64 65 66 67
// 24x2 = 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 13 14 15 16 17
//        40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57
// 40x2 = 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 13 14 15 16 17 18 19 1A 1B 1C 1D 1E 1F 20 21 22 23 24 25 26 27
//        40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 5B 5C 5D 5E 5F 60 61 62 63 64 65 66 67
// 40x4 = 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 13 14 15 16 17 18 19 1A 1B 1C 1D 1E 1F 20 21 22 23 24 25 26 27
//        40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 5B 5C 5D 5E 5F 60 61 62 63 64 65 66 67
//        00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 13 14 15 16 17 18 19 1A 1B 1C 1D 1E 1F 20 21 22 23 24 25 26 27
//        40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 5B 5C 5D 5E 5F 60 61 62 63 64 65 66 67
void LocateLCD(int x, int y)
{

#ifdef DISPLAY_8x1
	// 8x1
	PutCmdLCD(0x80 + x);
#endif
#ifdef DISPLAY_8x2
	// 8x2
	PutCmdLCD(0x80 + y*0x40 + x);
#endif

#ifdef DISPLAY_10x1
	// 10x1
	PutCmdLCD(0x80 + x);
#endif
#ifdef DISPLAY_10x4
	// 10x4
	switch(y){
	case 0:
		PutCmdLCD(0x80 + x);
		break;
	case 1:
		PutCmdLCD(0x80 + y*0x40 + x);
		break;
	case 2:
		PutCmdLCD(0x80 + y*0x0A + x);
		break;
	default:
		PutCmdLCD(0x80 + y*0x4A + x);
		break;
	}
#endif

#ifdef DISPLAY_16x1
	// 16x1
	if(x<8){
		PutCmdLCD(0x80 + x);
	}else{
		PutCmdLCD(0x80 + 0x40 + x-8);
	}
#endif
#ifdef DISPLAY_16x2
	// 16x2
	PutCmdLCD(0x80 + y*0x40 + x);
#endif
#ifdef DISPLAY_16x4
	// 16x4
	switch(y){
	case 0:
		PutCmdLCD(0x80 + x);
		break;
	case 1:
		PutCmdLCD(0x80 + y*0x40 + x);
		break;
	case 2:
		PutCmdLCD(0x80 + y*0x10 + x);
		break;
	default:
		PutCmdLCD(0x80 + y*0x50 + x);
		break;
	}
#endif

#ifdef DISPLAY_20x2
	// 20x2
	PutCmdLCD(0x80 + y*0x40 + x);
#endif
#ifdef DISPLAY_20x4
	// 20x4
	switch(y){
	case 0:
		PutCmdLCD(0x80 + x);
		break;
	case 1:
		PutCmdLCD(0x80 + y*0x40 + x);
		break;
	case 2:
		PutCmdLCD(0x80 + y*0x14 + x);
		break;
	default:
		PutCmdLCD(0x80 + y*0x54 + x);
		break;
	}
#endif

#ifdef DISPLAY_24x2
	// 24x2
	PutCmdLCD(0x80 + y*0x40 + x);
#endif

#ifdef DISPLAY_40x2
	// 40x2
	PutCmdLCD(0x80 + y*0x40 + x);
#endif
#ifdef DISPLAY_40x4
	switch(y){
	case 0,1:
		// 40x2 upper
		// Enable E1
		PutCmdLCD(0x80 + y*0x40 + x);
		break;
	case 2,3:
		// 40x2 lower
		// Enable E2
		PutCmdLCD(0x80 + (y-2)*0x40 + x);
		break;
	}
#endif


}


void PutsLCD(char *buffer)
{

	while(*buffer != '\0')
	{
		PutDatLCD(*buffer);
		buffer++;
	}

}




