//**********************
// CAN module Function for dsPIC30F4012
// 
// Condition:
// 
// (C)Copyright 2009 All rights reserved by Y.Onodera
// http://einst.hp.infoseek.co.jp
//**********************
#if defined(__dsPIC30F__)
#include <p30fxxxx.h>
#elif defined(__dsPIC33F__)
#include <p33Fxxxx.h>
#elif defined(__PIC24H__)
#include <p24Hxxxx.h>
#elif defined(__PIC24F__)
#include <p24Fxxxx.h>
#endif

#include "test.h"

#include <stdio.h>
#include "MSG.h"


void GetMSG(char msg[],int u)
{

	int i;
	unsigned int j,k;
	unsigned char s,a,b,c,d;

	// CAN:u=2, K:u=4
	s=buf[u];
	a=buf[u+1];
	b=buf[u+2];
	c=buf[u+3];
	d=buf[u+4];

	switch(s){
	case 0x00:
		PIDstatus00=(unsigned long)a*0x1000000 + (unsigned long)b*0x10000 + (unsigned long)c*0x100 + (unsigned long)d;
#ifdef TEST
		PIDstatus00=0xFFFFFFFF;
#endif
		j = a * 0x100 + b;
		k = c * 0x100 + d;
		sprintf(msg,"%04X%04X", j, k);
		break;
	case 0x20:
		PIDstatus20=(unsigned long)a*0x1000000 + (unsigned long)b*0x10000 + (unsigned long)c*0x100 + (unsigned long)d;
#ifdef TEST
		PIDstatus20=0xFFFFFFFF;
#endif
		j = a * 0x100 + b;
		k = c * 0x100 + d;
		sprintf(msg,"%04X%04X", j, k);
		break;
	case 0x40:
		PIDstatus40=(unsigned long)a*0x1000000 + (unsigned long)b*0x10000 + (unsigned long)c*0x100 + (unsigned long)d;
#ifdef TEST
		PIDstatus40=0xFFFFFFFF;
#endif
		j = a * 0x100 + b;
		k = c * 0x100 + d;
		sprintf(msg,"%04X%04X", j, k);
		break;
	case 0x01:
		j = a & 0x7F;
		sprintf(msg,"DTC# %3X", j);
		break;
	case 0x02:
		j = a * 0x100 + b;
		sprintf(msg,"DTC %04X", j);
		break;
	case 0x03:
		j = a * 0x100 + b;
		sprintf(msg,"FUEL%04X", j);
		break;
	case 0x04:
		j = a*100/255;
		sprintf(msg,"LOAD%3u%%", j);
		break;
	case 0x05:
		i = a - 40;
		sprintf(msg,"ECT %3dC", i);
		break;
	case 0x06:
		i = (a-128)*100/128;
		sprintf(msg,"SFT1%3d%%", i);
		break;
	case 0x07:
		i = (a-128)*100/128;
		sprintf(msg,"LFT1%3d%%", i);
		break;
	case 0x08:
		i = (a-128)*100/128;
		sprintf(msg,"SFT2%3d%%", i);
		break;
	case 0x09:
		i = (a-128)*100/128;
		sprintf(msg,"LFT2%3d%%", i);
		break;
	case 0x0a:
		j = a*3;
		sprintf(msg,"FRP%3ukP", j);
		break;
	case 0x0b:
		j = a;
		sprintf(msg,"MAP%3ukP", j);
		break;
	case 0x0c:
		j = a * 0x100 + b;
		j = j/4;
		sprintf(msg,"%4u RPM", j);
		break;
	case 0x0d:
		j = a;
		sprintf(msg,"%3u km/h", j);
		break;
	case 0x0e:
		i = a/2-64;
		sprintf(msg,"ADV  %3d", i);
		break;
	case 0x0f:
		i = a - 40;
		sprintf(msg,"IAT %3dC", i);
		break;
	case 0x10:
		j = a * 0x100 + b;
		j = j/100;
		sprintf(msg,"MAF  %3u", j);
		break;
	case 0x11:
		j = a *100 / 255;
		sprintf(msg,"TP  %3u%%", j);
		break;
	case 0x12:
		j = a;
		sprintf(msg,"AIR   %2X", j);
		break;
	case 0x13:
		j = a;
		sprintf(msg,"O2S   %2X", j);
		break;
	case 0x14:
	case 0x15:
	case 0x16:
	case 0x17:
	case 0x18:
	case 0x19:
	case 0x1a:
	case 0x1b:
		j = a*5;
		sprintf(msg,"O2S %4u", j);
		break;
	case 0x1c:
		j = a;
		sprintf(msg,"OBD   %2X", j);
		break;
	case 0x1d:
		j = a;
		sprintf(msg,"O2S   %2X", j);
		break;
	case 0x1e:
		j = a;
		sprintf(msg,"PTO   %2X", j);
		break;
	case 0x1f:
		j = a * 0x100 + b;
		sprintf(msg,"%5usec", j);
		break;
	case 0x21:
		j = a * 0x100 + b;
		sprintf(msg,"%5u km", j);
		break;
	case 0x22:
		j = a * 0x100 + b;
		j = j*0.079;
		sprintf(msg,"%4u kPa", j);
		break;
	case 0x23:
		j = a * 0x100 + b;
		sprintf(msg,"%5u0kP", j);
		break;
	case 0x24:
	case 0x25:
	case 0x26:
	case 0x27:
	case 0x28:
	case 0x29:
	case 0x2a:
	case 0x2b:
		j = a * 0x100 + b;
		j = j*0.0305;
		sprintf(msg,"EQ  %4u", j);
		break;
	case 0x2c:
		j = a*100/255;
		sprintf(msg,"EGR %3u%%", j);
		break;
	case 0x2d:
		i = a*100/128-100;
		sprintf(msg,"EGR %3d%%", i);
		break;
	case 0x2e:
		j = a*100/255;
		sprintf(msg,"EP  %3u%%", j);
		break;
	case 0x2f:
		j = a*100/255;
		sprintf(msg,"FLI %3u%%", j);
		break;
	case 0x30:
		j = a;
		sprintf(msg,"WARM %3u", j);
		break;
	case 0x31:
		j = a * 0x100 + b;
		sprintf(msg,"%5u km", j);
		break;
	case 0x32:
		i = a * 0x100 + b;
		i = i*0.25-8192;
		sprintf(msg,"%5d Pa", i);
		break;
	case 0x33:
		j = a;
		sprintf(msg," %3u kPa", j);
		break;
	case 0x34:
	case 0x35:
	case 0x36:
	case 0x37:
	case 0x38:
	case 0x39:
	case 0x3a:
	case 0x3b:
		j = a * 0x100 + b;
		j = j*0.0305;
		sprintf(msg,"EQ  %4u", j);
		break;
	case 0x3c:
	case 0x3d:
	case 0x3e:
	case 0x3f:
		j = a * 0x100 + b;
		i = j/10 - 40;
		sprintf(msg,"CT %4dC", i);
		break;
	case 0x41:
		j = a * 0x100 + b;
		k = c * 0x100 + d;
		sprintf(msg,"%04X%04X", j, k);
		break;
	case 0x42:
		j = a * 0x100 + b;
		j = j/1000;
		sprintf(msg,"PWR  %2uV", j);
		break;
	case 0x43:
		j = a * 0x100 + b;
		j = j*100/255;
		sprintf(msg,"  %5u%%", j);
		break;
	case 0x44:
		j = a * 0x100 + b;
		j = j*0.0305;
		sprintf(msg,"EQ  %4u", j);
		break;
	case 0x45:
		j = a*100/255;
		sprintf(msg,"TPR %3u%%", j);
		break;
	case 0x46:
		i = a-40;
		sprintf(msg,"AAT %3dC", i);
		break;
	case 0x47:
		j = a*100/255;
		sprintf(msg,"TPB %3u%%", j);
		break;
	case 0x48:
		j = a*100/255;
		sprintf(msg,"TPC %3u%%", j);
		break;
	case 0x49:
		j = a*100/255;
		sprintf(msg,"APPD%3u%%", j);
		break;
	case 0x4a:
		j = a*100/255;
		sprintf(msg,"APPE%3u%%", j);
		break;
	case 0x4b:
		j = a*100/255;
		sprintf(msg,"APPF%3u%%", j);
		break;
	case 0x4c:
		j = a*100/255;
		sprintf(msg,"TAC %3u%%", j);
		break;
	case 0x4d:
	case 0x4e:
		j = a * 0x100 + b;
		sprintf(msg,"%5umin", j);
		break;
	default:
		sprintf(msg,"unknown ");
		break;
	}

}


void GetDTC(char msg[],int u)
{

	unsigned char s,a,b,c,d;

	// CAN:u=2,4,6, K:u=4,6,8
	s=buf[u];
	a=buf[u+1];
	b=buf[u+2];
	c=buf[u+3];
	d=buf[u+4];

	switch(s&0xC0){
	case 0x00:
		sprintf(msg,"   P%02X%02X", s&0x3F, a);
		break;
	case 0x40:
		sprintf(msg,"   C%02X%02X", s&0x3F, a);
		break;
	case 0x80:
		sprintf(msg,"   B%02X%02X", s&0x3F, a);
		break;
	case 0xC0:
		sprintf(msg,"   U%02X%02X", s&0x3F, a);
		break;
	}

}


void PutMSG(int u)
{

	unsigned char s;

	// CAN:u=2, K:u=4
	s=buf[u];

	switch(s){
	case 0x00:
		buf[0]=0x06;
		buf[1]+=0x40;
		buf[3]=0xff;
		buf[4]=0xff;
		buf[5]=0xff;
		buf[6]=0xfe;
		break;
	case 0x20:
		buf[0]=0x06;
		buf[1]+=0x40;
		buf[3]=0xff;
		buf[4]=0xff;
		buf[5]=0xff;
		buf[6]=0xfe;
		break;
	case 0x40:
		buf[0]=0x06;
		buf[1]+=0x40;
		buf[3]=0xff;
		buf[4]=0xff;
		buf[5]=0xff;
		buf[6]=0xfe;
		break;
	case 0x01:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x02:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x03:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x04:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x05:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x06:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x07:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x08:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x09:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x0a:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x0b:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x0c:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x0d:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x0e:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x0f:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x10:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x11:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x12:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x13:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x14:
	case 0x15:
	case 0x16:
	case 0x17:
	case 0x18:
	case 0x19:
	case 0x1a:
	case 0x1b:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x1c:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x1d:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x1e:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x1f:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x21:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x22:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x23:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x24:
	case 0x25:
	case 0x26:
	case 0x27:
	case 0x28:
	case 0x29:
	case 0x2a:
	case 0x2b:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x2c:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x2d:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x2e:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x2f:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x30:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x31:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x32:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x33:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x34:
	case 0x35:
	case 0x36:
	case 0x37:
	case 0x38:
	case 0x39:
	case 0x3a:
	case 0x3b:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x3c:
	case 0x3d:
	case 0x3e:
	case 0x3f:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x41:
		buf[0]=0x06;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		buf[5]=0x05;
		buf[6]=0x0a;
		break;
	case 0x42:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x43:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x44:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	case 0x45:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x46:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x47:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x48:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x49:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x4a:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x4b:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x4c:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x05;
		break;
	case 0x4d:
	case 0x4e:
		buf[0]=0x04;
		buf[1]+=0x40;
		buf[3]=0x05;
		buf[4]=0x0a;
		break;
	default:
		buf[0]=0x03;
		buf[1]+=0x40;
		buf[3]=0x00;
		break;
	}

}
